"use client";

import { useSearchParams } from "next/navigation";
import { useCallback, useEffect, useState } from "react";
import { formatNumber } from "./utils/numberUtils";
import { getDaysSince } from "./utils/timeUtils";

const PROXY_URL = "https://proxy.fascinated.cc";
const SS_API_URL = PROXY_URL + "/https://scoresaber.com/api";
const SS_GET_PLAYERS_URL = SS_API_URL + "/players?page={}";
const SS_GET_PLAYER_DATA_FULL = SS_API_URL + "/player/{}/basic";

const PLAYERS_PER_PAGE = 50;

export default function Home() {
  const searchQuery = useSearchParams();
  const playerId = searchQuery.get("id");
  const rank = Number(searchQuery.get("rank"));
  const startedAt = searchQuery.get("started");

  const page = Math.max(Math.ceil(rank / PLAYERS_PER_PAGE), 1);

  const [mounted, setMounted] = useState(false);
  const [ppDiff, setPpDiff] = useState<number | undefined>(undefined);

  const updateRank = useCallback(async () => {
    if (!playerId || !rank || !page) return;
    console.log("Updating rank...");
    const playerData = await fetch(
      SS_GET_PLAYER_DATA_FULL.replace("{}", playerId)
    ).then((res) => res.json());

    const playerAtRank = await fetch(
      SS_GET_PLAYERS_URL.replace("{}", page.toString())
    )
      .then((res) => res.json())
      .then((res) => res.players.find((player: any) => player.rank === rank));

    if (!playerAtRank) {
      console.log("Player not found");
      return;
    }

    const ppDiff = playerAtRank.pp - playerData.pp;
    setPpDiff(ppDiff);
  }, [page, playerId, rank]);

  useEffect(() => {
    if (mounted) return;
    setMounted(true);

    updateRank();
    setInterval(updateRank, 1000 * 60 * 1); // update every minute
  }, [mounted, updateRank]);

  if (!playerId || !rank) {
    return (
      <main className="text-xl p-2 text-black">
        <p>
          Usage:{" "}
          <span className="text-blue-500">
            <a href="https://git.fascinated.cc/Fascinated/scoresaber-pp-to-rank-overlay#usage">
              https://git.fascinated.cc/Fascinated/scoresaber-pp-to-rank-overlay#usage
            </a>
          </span>
        </p>
      </main>
    );
  }

  if (!ppDiff) {
    return (
      <main className="text-xl p-2 text-white">
        <p>Calculating...</p>
      </main>
    );
  }

  return (
    <main className="text-xl text-white">
      <div className="flex gap-1 flex-col p-2">
        <p>
          {formatNumber(ppDiff, 2)}pp until rank {formatNumber(rank)}
        </p>
        {startedAt && (
          <p>
            Day {formatNumber(getDaysSince(new Date(Number(startedAt))) + 1)}
          </p>
        )}
      </div>
    </main>
  );
}
